#!/bin/bash

set -e
set -x

check_link_status() {
    local iface="$1"
    ethtool "$iface" 2>/dev/null | grep -q "Link detected: yes"
}

check_speed() {
    local iface="$1"
    local speed=$(ethtool "$iface" 2>/dev/null | grep "Speed:" | awk '{print $2}' | sed 's/Mb\/s//')
    [[ "$speed" == "1000" ]]
}

get_ip_address() {
    local iface="$1"
    ip -4 addr show "$iface" | awk '/inet / {print $2}' | cut -d/ -f1
}

is_valid_ip() {
    local ip="$1"
    if [[ -z "$ip" ]]; then
        return 1
    fi
    if [[ "$ip" =~ ^169\.254\..* || "$ip" == "0.0.0.0" || "$ip" == "127.0.0.1" ]]; then
        return 1
    fi
    return 0
}

is_dhcp_ip() {
    local iface="$1"
    dhcpcd -U "$iface" 2>/dev/null | grep -q "dhcp_server_identifier"
}

dhcp_test() {
    echo "Checking DHCP status for end0 and end1..."

    nmcli connection delete static-end1 || true

    for iface in end0 end1; do
        echo "Checking network cable for $iface..."

        # Wait up to 30 seconds for the network cable to be detected
        TIMEOUT=30
        while ! check_link_status "$iface"; do
            echo "$iface: No cable detected, waiting..."
            sleep 1
            ((TIMEOUT--))
            if [ $TIMEOUT -eq 0 ]; then
                echo "$iface: DHCP failed (No cable detected)"
                exit 1
            fi
        done
        echo "$iface: Cable connected"

        # Check if the negotiated speed is 1000Mb/s
        if ! check_speed "$iface"; then
            echo "$iface: Negotiated speed is NOT 1000Mb/s, test failed"
            exit 1
        fi
        echo "$iface: Negotiated speed is 1000Mb/s"

        # Release the old IP and request a new DHCP IP
        echo "Requesting DHCP IP for $iface..."
        dhcpcd -k "$iface" || true
        dhcpcd "$iface" || true

        # Wait up to 30 seconds to acquire an IP address
        TIMEOUT=30
        while [ $TIMEOUT -gt 0 ]; do
            IP_ADDR=$(get_ip_address "$iface")
            if is_valid_ip "$IP_ADDR" && is_dhcp_ip "$iface"; then
                echo "$iface: DHCP pass, IP acquired: $IP_ADDR"
                break
            fi
            ((TIMEOUT--))
            sleep 1
        done

        # Timeout failure
        if [ $TIMEOUT -eq 0 ]; then
            echo "$iface: DHCP failed (Invalid or Non-DHCP IP)"
            exit 1
        fi
    done

    echo "DHCP check completed successfully."
}

clean_up() {
    echo "Executing clean operations..."

    nmcli connection delete static-end1 || true
    rm -rf /etc/systemd/system/serial-getty@ttyS0.service.d
    rm -rf /etc/systemd/system/es_con_srv.service
    rm -rf /usr/local/bin/es_con_srv
    systemctl daemon-reload
    rm -rf /usr/local/bin/station_status
    systemctl stop systemd-journald
    rm -rf /var/log/*
    journalctl --vacuum-time=1s
    rm -rf /tmp/*
    dmesg -C
    rm -rf ~/.cache/* ~/.local/share/*
    cat /dev/null > ~/.bash_history
    apt clean
    history -c
    rm -- "$0"

    exit 0
}

if [ "$1" == "dhcp" ]; then
    dhcp_test
    clean_up
fi

if [ "$1" == "clean" ]; then
    clean_up
fi

if [ -z "$1" ]; then
    echo "Usage: $0 <ipaddr> [macaddr] | clean"
    exit 1
fi

ipaddr="$1"
macaddr="$2"

while ! ethtool end1 | grep -q "Link detected: yes"; do
    echo "waiting for end1 link up..."
    sleep 2
done

gateway="$(echo $ipaddr | awk -F'.' '{print $1"."$2"."$3".1"}')"

systemctl daemon-reload

echo 3 > /proc/sys/kernel/printk

if [ -n "$macaddr" ]; then
    echo "Changing MAC address of end1 to $macaddr"
    ip link set dev end1 down
    ip link set dev end1 address "$macaddr"
    ip link set dev end1 up
fi

nmcli connection delete static-end1 || true

nmcli connection add type ethernet ifname end1 con-name static-end1 ip4 ${ipaddr}/24 gw4 ${gateway}

nmcli connection up static-end1
